import serial
import time
import re
import numpy as np
from matplotlib import cm
from PIL import Image
from scipy.ndimage import gaussian_filter as gf
import scipy.signal
from matplotlib import pyplot as plt

def print_bts(bts):
    for n in bts:
        print("{0:0{1}X}".format(n,2), end = " ")
    print()

def split_frame(frame):
    len_frm = len(frame)
    DISTANCE_BINS = 34
    SWEEPS_PER_FRAME = 64
    
    
    frame_arr = np.empty((DISTANCE_BINS,SWEEPS_PER_FRAME))
    
    for i in range(0,len_frm,2):
        v = int.from_bytes(frame[i:i+2], "little")
        entry = i//2
        y = entry//DISTANCE_BINS
        x = entry - y*DISTANCE_BINS
        # print("y: ",y," x: ",x)
        frame_arr[x,y] = v
    return frame_arr

def get_vels_from_frame(frame, pb_test):
    bins,sweeps = frame.shape
    
    
    frame = gf(frame,1.5)
        
    mn,mx = np.min(frame),np.max(frame)
    im = Image.fromarray(np.uint8(cm.viridis((frame-mn)/(mx-mn))*255))
    im.save("conv.png")
    
    maxbin,maxsweep = np.unravel_index(frame.argmax(), frame.shape)
    print((maxbin,maxsweep))
    # for v in frame[maxbin]:
        # print(v)

    fig, ax = plt.subplots()
    ax.plot(frame[maxbin])
    ax2 = ax.twinx()
    ax2.plot(pb_test, c = 'r')
    
    
    plt.savefig('plot.png')
    plt.close()

    mx = 0;
    for j in range(sweeps//2):
        d = frame[maxbin][j];
        mx = mx if mx > d else d;

    mx /= 3;
    mass = 0;
    centre = 0;
    for j in range(sweeps//2):
        d = frame[maxbin][j];
        d = d if d > mx else 0;
        mass +=  d;
        centre += d*j;

    centre /= mass;

    vel = centre * (500E3/64) * 2.445e-3
    dist = 0.06*1000*maxbin + 0.24*1000
    print('ftp vel:' + str(vel))
    print('ftp dist:' + str(dist))
    return (vel,dist)


print('Checking Sensor')

while True:
    state = 0
    bts = b''
    raw = b''
    peak_bins = []
    ser = serial.Serial('COM28', 9600, timeout = 3)
    tik = time.time()
    while(time.time() - tik < 12):
        bt = ser.read()
        if (state != 2):
            try:
                print(bt.decode("utf-8"), end = '',  flush=True)
            except UnicodeDecodeError:
                print(bt, end = '')
        
        if(state == 0 and bt == b'R'):
            print()
            print('Sensor Ready')
            state = 1
            ser.write(b'XA')
        elif(state == 1 and bt == b':'):
            chunks = int.from_bytes(ser.read(), 'little')+1
            print("Chunks: " + str(chunks))
            state = 2
            raw = b''
            ser.write(b'A')
        elif(state == 2 and len(raw) < chunks*256):
            raw += bt
            ser.write(b'A')
            if(len(raw) >= chunks * 256):
                state = 4
                print('end raw')
        elif(state == 4):
            bts += bt
    
    while(ser.in_waiting):
        ser.read()
    ser.close()
    rspstr = bts.decode("utf-8")
    vals = rspstr.split('\n')[1:]
    
    
    frame = split_frame(raw[:-256])
    mn,mx = np.min(frame),np.max(frame)
    im = Image.fromarray(np.uint8(cm.viridis((frame-mn)/(mx-mn))*255))
    im.save("frame.png")

    # print(frame)
    ftpv, ftpd = get_vels_from_frame(frame, peak_bins)
    
    try:

        vel = vals[0].split()[1]
        dist = vals[1].split()[1]
        amp = vals[2].split()[1]
        spread = vals[3].split()[1]
        angle = vals[4].split()[1]
        peaks = vals[5].split()[1]

        peaks = peaks.split('|')
        peaks = [int(x, 16) for x in peaks]
        pdist = [x >> 16 for x in peaks]
        pamps = [x & 0xFFFF for x in peaks]
        

        print('========= RESULTS ========')
        print('velocity: ', vel, ' mm/s ftp: ', ftpv)
        print('distance: ', dist, ' mm ftp: ', ftpd)
        print('amplitude: ', amp)
        print('spread: ', spread)
        print('angle: ', angle, ' deg')
        for a,d in zip(pamps,pdist):
            print('peak: ', d, "mm  | amplitude: ", a)
        print()
    
    except IndexError:
        pass
        
    

    
    print('Resetting')
    print('============================')


