#include "src\RadarService.h"
#include <Wire.h>


#define ACCADR 0x0E

#define PEAK_THRESHOLD 0.5
#define SWEEP_RATE	500E3   //(mHz) //this needs to be changed depending on instalation angle & expected flow
#define DISTANCE_RES 60	    //(mm)
#define SWEEP_LENGTH  2000  //(mm) //this needs to be changed depending expected water level range
#define SWEEP_START  240    //(mm) //this needs to be changed depending on installation height
#define SWEEPS 64

RadarService Radar(PEAK_THRESHOLD, SWEEP_RATE, DISTANCE_RES, SWEEP_LENGTH, SWEEPS); 

uint16_t velocity;
uint16_t distance;
uint16_t amplitude;
uint16_t spread;
uint32_t peaks[4];
uint16_t data[128];

void setup(){
	
	
	Serial.begin(9600);
	Serial.println(F("Begin"));
	
	Serial.flush();
	
	//turn on accelerometer
	pinMode(A1, OUTPUT);
	digitalWrite(A1, HIGH);
	
	Wire.begin();
	delay(10);
	Radar.power_on();
	Radar.init();
	
	
	

}


void loop(){
    delay(10);
	Serial.write('R');
	uint8_t mode = get_mode();
    if(mode){
        
        get_radar(SWEEP_START, mode);

        if(get_ack('A')){
            Serial.print(F("vel: "));
            Serial.println(velocity);
            Serial.print(F("dist: "));
            Serial.println(distance);
            Serial.print(F("amp: "));
            Serial.println(amplitude);
            Serial.print(F("spread: "));
            Serial.println(spread);
            Serial.print(F("angle: "));
            Serial.println(read_acc_angle());
            Serial.print(F("peaks: "));
			Serial.print(peaks[0], HEX);
            Serial.print('|');
			Serial.print(peaks[1], HEX);
            Serial.print('|');
			Serial.print(peaks[2], HEX);
            Serial.print('|');
			Serial.println(peaks[3], HEX);
        }
        
        Serial.println();
	}	
	Serial.flush();
	delay(10);
	Radar.power_off();
	digitalWrite(A1, LOW);
	Serial.end();
	
	sleepMode(SLEEP_POWER_DOWN);
	sleep();
	

}

void dump_data(void){
	while(Serial.available()){
		Serial.read();
	}
	Serial.print("D:");
	//chunksize 256 bytes;
	const uint16_t chunksize = 256;
	uint16_t datalen = Radar.read_reg(0x83);//get data length
	uint8_t chunks = (sizeof(uint16_t)*datalen)/chunksize;
	Serial.write(chunks);
	for(uint8_t i = 0; i< chunks+1; i++){
		if(not get_ack('A')){break;}
		
		Radar.read_buf_chunk( data, i*chunksize);
		Serial.write((uint8_t*) data, chunksize);
		
		
		while(Serial.available()){Serial.read();}
	}
	Serial.print('\n');
}

uint8_t get_mode(void){
	
	uint32_t tik = millis();
	while(millis() - tik < 100){
		if(Serial.available()){
			char chr = Serial.read();
			if(chr == 'P'){return 1;}
			if(chr == 'V'){return 2;}
			if(chr == 'M'){return 3;}
			if(chr == 'D'){return 4;}
			if(chr == 'Q'){return 5;}
			if(chr == 'X'){return 6;}
			if(chr == 'E'){return 7;}
		}
	}
	return 0;
}

uint8_t get_ack(char ack_char){
	
	uint32_t tik = millis();
	while(millis() - tik < 2000){
		if(Serial.available()){
			char chr = Serial.read();
			if(chr == ack_char){return 1;}
			// if(chr == 'N'){return -1;}
		}
	}
	return 0;
}


void get_radar(uint16_t dist, uint8_t mode){
    
    if(mode == 1){
        Radar.measure_sparse(dist);
        Radar.eval_sparse(&velocity, &distance, &amplitude, &spread);
    }else if (mode == 2){
        Radar.measure_sparse(dist);
        dump_data();
        Radar.eval_sparse(&velocity, &distance, &amplitude, &spread); 
		Radar.measure_envelope(dist);
        Radar.eval_envelope(peaks);			
    }else if(mode == 3){
		Radar.measure_sparse_av(dist, 6);	
        Radar.est_sparse(&velocity, &distance, &amplitude, &spread);
    }else if(mode == 4){
		Radar.measure_sparse(dist);
        Radar.eval_sparse(&velocity, &distance, &amplitude, &spread);
		Radar.measure_envelope(dist);
        Radar.eval_envelope(peaks);
	}else if(mode == 5){
		Radar.measure_sparse_av(dist, 6);
        Radar.est_sparse(&velocity, &distance, &amplitude, &spread);
		
		Radar.measure_envelope(dist);
        Radar.eval_envelope(peaks);
		
	}else if(mode == 6){
		Radar.measure_sparse_av(dist, 6);
		dump_data();
        Radar.est_sparse(&velocity, &distance, &amplitude, &spread);
				
	}else if(mode == 7){
		Radar.measure_envelope(dist);
        Radar.eval_envelope(peaks);	
	}
}


/////////////////accelerometer i2c
uint8_t readreg(uint8_t reg){
	Wire.beginTransmission(ACCADR);
	Wire.write(reg);
	Wire.endTransmission(false);
	
	Wire.requestFrom(ACCADR,1);
	return Wire.read();
}

uint8_t writereg(uint8_t reg, uint8_t val){
	Wire.beginTransmission(ACCADR);
	Wire.write(reg);
	Wire.write(val);
	Wire.endTransmission();
}

float get_angle(){
	delay(20);
	float xg = (readreg(0x07)<<8) + readreg(0x06);
	float yg = (readreg(0x09)<<8) + readreg(0x08);
	float zg = (readreg(0x0B)<<8) + readreg(0x0A);
	float pg = sqrt(xg*xg + yg*yg);
	float cs = atan2(zg,pg);
	return cs*57.295f;
}

float read_acc_angle(){
	float dec;
	
	writereg(0x1B, 0b11000000);

	if(readreg(0x1B) == 0b11000000){
		dec = get_angle();
	}
	writereg(0x1B, 0b01000000);
	
	return dec;
}
