#include "RadarService.h"

RadarService::RadarService(	float pt,
							uint32_t sr,
							uint8_t  dr,
							uint32_t sl,
							uint8_t  sp
){
									
	peak_threshold = pt;
	sweep_rate = sr;
	distance_res = dr;
	sweep_length = sl;
	sweeps = sp;
	distance_bins = sl/dr + 1;
}

uint8_t RadarService::get_byte(uint32_t val, uint8_t byte){
	return (val & (0xFFL << (8*byte))) >> (8*byte);
}

void RadarService::power_on(void){
	//turn on the radar
	pinMode(6,OUTPUT);
	digitalWrite(6,HIGH);
	delay(10);
	//turn on the radar
	pinMode(7,OUTPUT);
	digitalWrite(7,HIGH);
	delay(10);
	//set wakup high
	pinMode(5,OUTPUT);
	digitalWrite(5,HIGH);
	
	//reset the radar
	pinMode(4,OUTPUT);
	digitalWrite(4,LOW);
	delay(1);
	digitalWrite(4,HIGH);
	delay(20);
}

void RadarService::init(void){
	Serial1.begin(115200);
	write_reg(0x07,57600, false);
	Serial1.flush();
	delay(2);
	Serial1.end();
	delay(2);
	Serial1.begin(57600);
	//new baudrate
	Serial.println(read_reg(0x07));
}

void RadarService::power_off(void){
	//turn on the radar
	pinMode(6,OUTPUT);
	digitalWrite(6,LOW);
	pinMode(7,OUTPUT);
	digitalWrite(7,LOW);
	pinMode(5,INPUT);//WAKEUP
	pinMode(4,INPUT);//RST
	pinMode(11,INPUT);//TxRx
	pinMode(12,INPUT);//TxRx
}

uint32_t RadarService::read_reg(uint8_t reg){
	//0xCC, 0x01, 0x00, 0xF8, reg, 0xCD
	clr_cmd();

	while(Serial1.available()){Serial1.read();}

	cmd_str[0] = 0xCC;
	cmd_str[1] = 0x01;
	cmd_str[2] = 0x00;
	cmd_str[3] = 0xF8;
	cmd_str[4] = reg;
	cmd_str[5] = 0xCD;
	send(6);

	
	uint32_t value;
	value = get_response();
                        // Serial.print("reg: ");
                        // Serial.print(reg, HEX);
                        // Serial.print(" val: ");
                        // Serial.print(value, HEX);
                        // Serial.println();
	return value;
}

uint32_t RadarService::get_response(void){
	int16_t j = 0;
	
	uint32_t value = 0;
	int16_t bt;
	
	do{

		bt = read_next();
		if (bt == -2){
			j = 0;
			continue;
		}
		
		
		if (0 <= bt and bt <= 255){
		uint32_t nbt = bt;
		nbt = nbt << (j%4)*8;
		value |= nbt;
		}
		
		
		j++;

	}while(bt != -1);
                                    // Serial.print("reg val: ");
                                    // Serial.println(value);
	delay(1);
	return value;
}

void RadarService::read_buf_chunk(uint16_t* data, uint16_t chunk){
	//0xCC, 0x03, 0x00, 0xFA, 0xE8, 0x00, 0x00, 0xCD
	clr_cmd();
	
	while(Serial1.available()){Serial1.read();}
	
	cmd_str[0] = 0xCC;
	cmd_str[1] = 0x03;
	cmd_str[2] = 0x00;
	cmd_str[3] = 0xFA;
	cmd_str[4] = 0xE9;
	cmd_str[5] = get_byte(chunk, 0);
	cmd_str[6] = get_byte(chunk, 1);
	cmd_str[7] = 0xCD;
	
	send(8);
	
	int16_t j = 0;
	
	int32_t value;
	int32_t bt = 0;
	do{
		bt = read_next();
		
		if (bt == -2){
			j = 0;
			continue;
		}
		
		if (j%2 == 0){
			value = 0;
		}
		
		if (0 <= bt and bt <= 255){
			value |= bt << (j%2)*8;
			
			if (j%2 == 1){
				data[j/2] = value;
			}
		}
		j++;

	}while(bt != -1);
	
}

void RadarService::measure_sparse_av(int32_t sd, uint8_t av){
	//open sensor and first measurement
	measure_sparse(sd);
	
	//clear accumulant frame
	write_reg(0xDB, 0x00000000);
	write_reg(0xDA, 0x00000001);
	write_reg(0x03, 0x07);
	
	for(uint8_t i = 0; i < av; i++){
		//data fft
		write_reg(0xD6, 0b000011);
		write_reg(0x03,0x06);
		
		//pack into accumulant
		uint32_t mscl_reg = read_reg(0xD2);
		float mscl = *(float*)&mscl_reg;
		mscl = 1/mscl;
		write_reg(0xDB, *(uint32_t*)&mscl);
		write_reg(0xDA, 0x00000006);
		write_reg(0x03,0x07);

		//next measuremnt
		if(i < (av - 1)){
			read_reg(0x06);
			write_reg(0x03,0x05);
		}	
	}
	//sqrt and load back into data
	write_reg(0xDA, 0x00000097);
	write_reg(0x03,0x07);
}

void RadarService::measure_sparse(int32_t sd){
	start_distance = sd;
	//close sensor
	write_reg(0x03,0x00);
	//clear status
	write_reg(0x03,0x04);
	read_reg(0x06);
	//sparse mode
	write_reg(0x02,0x04);
	//close sensor
	write_reg(0x03,0x00);
	read_reg(0x06);
	//sparse mode
	write_reg(0x02,0x04);

	//set no software upper limit on rate we can poll sensor
	write_reg(0x23,0);
	//set power mode to active
	write_reg(0x25,0x03);
	//set profile to 3
	write_reg(0x28,0x03);
	//set sparse sampling mode to A
	write_reg(0x42,0x00);
	//set repition mode to be limited to update rate
	write_reg(0x22,0x02);
	//set sweep rate to 1 000 000 mHz
	write_reg(0x41, sweep_rate);


	//set start distace to 240 mm
	write_reg(0x20,start_distance);
	//set length to 560 mm
	write_reg(0x21,sweep_length);

	//set gain to 500
	write_reg(0x24,500);
	//set downsampling factor to 1
	write_reg(0x29,1);
	// // // // // // // // //set HW_ACC_AVE_SAMPLES to 27
	write_reg(0x30,50);
	//set sweeps per frame to 64
	write_reg(0x40, sweeps);
	//disable attenuation
	write_reg(0x32, 0x00);
	//enable TX
	write_reg(0x26, 0x00);


	//enable asynchonus measurement
	write_reg(0x33, 0x01);
	//set power mode to active
	write_reg(0x25, 0x03);
	//enable uart streaming
	write_reg(0x05, 0x00);
	//create service
	write_reg(0x03, 0x01);

	///////////////////////ADD CHECK CODE HERE///////////////////
	// //check state
	// read_reg(0x06);

	// //read start
	// read_reg(0x81);
	// //read len
	// read_reg(0x82);
	// //read data length
	// read_reg(0x83);
	// //read sweep rate
	// read_reg(0x84);
	// //read step length
	// read_reg(0x85);

	//active service
	write_reg(0x03,0x02);
	
	//block untill data taken
	// while(not (read_reg(0x06) & 0x00000100));
	// delay(1000);
	read_reg(0x06);

	write_reg(0x03,0x05);
}

void RadarService::measure_envelope(int32_t sd){
	start_distance = sd;
	//close sensor
	write_reg(0x03,0x00);
	//clear status
	write_reg(0x03,0x04);
	read_reg(0x06);
	//sparse mode
	write_reg(0x02,0x02);
	//close sensor
	write_reg(0x03,0x00);
	read_reg(0x06);
	//sparse mode
	write_reg(0x02,0x02);

	//set no software upper limit on rate we can poll sensor
	write_reg(0x23,0);
	//set power mode to active
	write_reg(0x25,0x03);
	//set profile to 1
	write_reg(0x28,0x01);
	//set sparse sampling mode to A
	write_reg(0x42,0x00);
	//set repition mode to be limited to update rate
	write_reg(0x22,0x02);

	//set start distace to 240 mm
	write_reg(0x20,start_distance);
	//set length to 560 mm
	write_reg(0x21,sweep_length);

	//set gain to 500
	write_reg(0x24,500);
	//set downsampling factor to 4
	write_reg(0x29,4);
	// // // // // // // // //set HW_ACC_AVE_SAMPLES to 27
	write_reg(0x30,60);
	//set running average decay to 0.8
	write_reg(0x40, 800);
	//disable attenuation
	write_reg(0x32, 0x00);
	//enable TX
	write_reg(0x26, 0x00);
	//set maximum unambigious range to 6m
	write_reg(0X34, 0x06);

	//set number of sparse repeat measurements
    write_reg(0x41, 6);

	//enable asynchonus measurement
	write_reg(0x33, 0x01);
	//set power mode to active
	write_reg(0x25, 0x03);
	//enable uart streaming
	write_reg(0x05, 0x00);
	//create service
	write_reg(0x03, 0x01);

	///////////////////////ADD CHECK CODE HERE///////////////////
	// //check state
	// read_reg(0x06);

	// //read start
	// read_reg(0x81);
	// //read len
	// read_reg(0x82);
	// //read data length
	// read_reg(0x83);
	// //read sweep rate
	// read_reg(0x84);
	// //read step length
	// read_reg(0x85);

	//active service
	write_reg(0x03,0x02);
	
	//block untill data taken
	// while(not (read_reg(0x06) & 0x00000100));
	// delay(1000);
	read_reg(0x06);

	write_reg(0x03,0x05);
}

void RadarService::eval_sparse(uint16_t* vel, uint16_t* dist, uint16_t* amp, uint16_t* sprd){
	
	//results registers
	//0xD0 :: Velocity (mm/s)
	//0xD1 :: Distance (mm)
	//0xD2 :: Amplitude (arb)
	//0xD3 :: Mean Square Distance (arb)
	
	//option registers
	//0xD4 :: Mean Square Distance threshold (x1000)
	//0xD5 :: Gaussian Kernal StDev (x1000)
	//0xD6 :: Data Eval Mode
	//0xD7 :: Focus weight Radius
	//0xD8 :: Data zeroing threshold
	write_reg(0xD5,1500);
	write_reg(0xD6, 0b01111);
	write_reg(0xD7, 2);
	write_reg(0xD8, 600);
	
	//do eval?
	write_reg(0x03,0x06);
	
	//read results
	*vel = read_reg(0xD0);
	*dist = read_reg(0xD1);
	*amp = read_reg(0xD2);
	*sprd = read_reg(0xD3);
	
}

uint8_t RadarService::pre_proc_sparse(uint16_t* bins, float* acc_bins){
	
	//results registers
	//0xD0 :: Velocity (mm/s)
	//0xD1 :: Distance (mm)
	//0xD2 :: Amplitude (arb)
	//0xD3 :: Mean Square Distance (arb)
	
	//option registers
	//0xD4 :: Mean Square Distance threshold (x1000)
	//0xD5 :: Gaussian Kernal StDev (x1000)
	//0xD6 :: Data Eval Mode
	//0xD7 :: Focus weight Radius
	//0xD8 :: Data zeroing threshold
	write_reg(0xD5,1500);
	write_reg(0xD6, 0b00001111);
	write_reg(0xD7, 0);
	// write_reg(0xD8, 600);
	// write_reg(0xD9, 0x00000001);
	
	//do eval?
	write_reg(0x03,0x06);
	
	uint8_t dist = (read_reg(0xD1)-start_distance)/distance_res;

		
	// uint16_t ofset = dist*sweeps*2;
	for(uint8_t i = 0; i < sweeps/2; i++){
        read_buf_chunk(bins, i*distance_bins*2);
        acc_bins[i] += ((float)bins[dist])*bins[dist];
        delay(1);
    }
    return dist;
	
}

void RadarService::est_sparse(uint16_t* vel, uint16_t* dist, uint16_t* amp, uint16_t* sprd){
	
	//results registers
	//0xD0 :: Velocity (mm/s)
	//0xD1 :: Distance (mm)
	//0xD2 :: Amplitude (arb)
	//0xD3 :: Mean Square Distance (arb)
	
	//option registers
	//0xD4 :: Mean Square Distance threshold (x1000)
	//0xD5 :: Gaussian Kernal StDev (x1000)
	//0xD6 :: Data Eval Mode
	//0xD7 :: Focus weight Radius
	//0xD8 :: Data zeroing threshold
	write_reg(0xD5,1500);
	write_reg(0xD6, 0b0100001100);
	write_reg(0xD7, 333);
	
	//do eval?
	write_reg(0x03,0x06);
	
	*vel = read_reg(0xD0);
	*dist = read_reg(0xD1);
	*amp = read_reg(0xDB);
	*sprd = read_reg(0xD3);

}

void RadarService::eval_envelope(uint32_t peaks[4]){
	
	//results registers
	//0xD0 :: peak 1
	//0xD1 :: peak 2
	//0xD2 :: peak 3
	//0xD3 :: peak 4
	
	//option registers
	//0xD4 :: min peak seperation
	write_reg(0xD4,200);

	//do eval?
	write_reg(0x03,0x06);
	
	//read results
	peaks[0] = read_reg(0xD0);
	peaks[1] = read_reg(0xD1);
	peaks[2] = read_reg(0xD2);
	peaks[3] = read_reg(0xD3);
	
}


void RadarService::send(uint8_t len){
	// Serial.print("S: ");
	// for (uint8_t k; k<len; k++){
		// Serial.print(cmd_str[k], HEX);
		// Serial.print(' ');
	// }
	// Serial.println();
	Serial1.write(cmd_str, len);
}

void RadarService::clr_cmd(){
	memset(cmd_str, '\0', 10);
}


uint8_t RadarService::read(){
	while (not Serial1.available()){}
	uint8_t bt = Serial1.read();
	// Serial.print("R: ");
	// Serial.println(bt,HEX);
	return bt;
}

int16_t RadarService::read_next(){

	if(i == end){
		end = -1;
		i = 0;
		length = 0;
		return -1;
	}
	uint8_t bt = read();
	
	// Serial.println(bt, HEX);
	//set the length of data
	if (i == 1){length =  bt;}
	if (i == 2){length |=  (uint16_t)bt << 8; end = length + 4;}
	if(4 < i and i < end){
		i++;
		return bt;	
	}else{
		i++;
		return -2;
	}
	
}

void RadarService::write_reg(uint8_t reg, uint32_t val){
	write_reg(reg,val,true);
}

void RadarService::write_reg(uint8_t reg, uint32_t val, bool block){
	
	
	//0xCC, 0x05, 0x00, 0xF9, reg, get_byte(val,0), get_byte(val,1), get_byte(val,2), get_byte(val,3), 0xCD
	
	clr_cmd();
	
	while(Serial1.available()){Serial1.read();}
	
	cmd_str[0] = 0xCC;
	cmd_str[1] = 0x05;
	cmd_str[2] = 0x00;
	cmd_str[3] = 0xF9;
	cmd_str[4] = reg;
	cmd_str[5] = get_byte(val,0);
	cmd_str[6] = get_byte(val,1);
	cmd_str[7] = get_byte(val,2);
	cmd_str[8] = get_byte(val,3);
	cmd_str[9] = 0xCD;
	send(10);

	
	if (block){
		uint32_t val = get_response();
                // Serial.print("reg: ");
                // Serial.print(reg, HEX);
                // Serial.print(" val: ");
                // Serial.print(val, HEX);
                // Serial.println();
	}
}