#include <stdint.h>
#include <Arduino.h>

class RadarService{
	public:
		RadarService(float pt, uint32_t sr, uint8_t dr, uint32_t sl, uint8_t sp);
		
		void power_on(void);
		void init(void);
		void power_off(void);
		
		int16_t read_next(); 
		uint32_t read_reg();
		void read_buf_chunk(uint16_t* , uint16_t);
		void write_reg(uint8_t, uint32_t, bool);
		void write_reg(uint8_t, uint32_t);
		uint32_t read_reg(uint8_t);
		uint32_t get_response(void);
		void measure_sparse(int32_t);
		void measure_sparse_av(int32_t, uint8_t);
		void measure_envelope(int32_t);
		
		void eval_sparse(uint16_t*, uint16_t*, uint16_t*, uint16_t*);
		void eval_envelope(uint32_t[4]);
		uint8_t pre_proc_sparse(uint16_t* bins, float* acc_bins);
		void est_sparse(uint16_t*, uint16_t*, uint16_t*, uint16_t*);
		
		float peak_threshold;
		uint32_t sweep_rate;
		uint8_t  distance_res;
		uint32_t sweep_length;
		uint8_t  sweeps;
		uint8_t  distance_bins;
		int32_t start_distance;

	private:
	
		int16_t end = -1;
		int16_t i = 0;
		int16_t length;
		uint8_t read();
		uint8_t cmd_str[10];
		void send(uint8_t);
		void clr_cmd();
		uint8_t get_byte(uint32_t, uint8_t);
};