import numpy as np
from matplotlib import pyplot as plt
from scipy.stats import linregress

def line(l, x):
    return l.slope*x + l.intercept

#########depth##################################################

data = np.loadtxt('depth_data.csv', delimiter = ',', skiprows = 1)

depth = data[:, 0]
dA = data[:, 1]
dB = data[:, 2]
dC = data[:, 3]

plt.figure(figsize=(7.16, 6), dpi = 94)

plt.rcParams.update({'font.size': 12})
plt.rc('axes', axisbelow=True)
plt.grid()

series = [(dA, 'forestgreen', 's', r'A: R$^2$=%0.2f, y=%0.2fx %+0.2f'),
          (dB, 'royalblue', 'o', r'B: R$^2$=%0.2f, y=%0.2fx %+0.2f'),
          (dC, 'orange', '^', r'C: R$^2$=%0.2f, y=%0.2fx %+0.2f'),]

uncy = uncx = len(depth)*[2]

for d,c,m,l in series:
    lobf = linregress(depth,d)
    print(l, lobf)
    
    plt.plot([0,200], lobf.slope*np.array([0,200])+lobf.intercept, color = c)

    plt.errorbar(depth, 
                    d, 
                    xerr = uncx, 
                    yerr = uncy,
                    fmt = ',',
                    ecolor = 'k',
                    elinewidth = 0.2,
                    capsize = 2.0,
                    color = c, 
                    marker = m, 
                    label = l % (lobf.rvalue**2, lobf.slope, lobf.intercept))

plt.legend()
plt.xlabel("Measured Depth (mm)")
plt.ylabel("Radar Depth (mm)")
plt.xlim([0,140])
plt.ylim([0,180])
plt.savefig('depth_scatter.png', dpi = 300, transparent = True)
plt.show() 

########velocity##################################################

data = np.loadtxt('vel_data.csv', delimiter = ',', skiprows = 1)

ven = data[:, 0]
vA = data[:, 2]
vB = data[:, 3]
vC = data[:, 4]
uncx = data[:,1]
uncy = data[:,5]

plt.figure(figsize=(7.16, 6), dpi = 94)

plt.rcParams.update({'font.size': 12})
plt.rc('axes', axisbelow=True)
plt.grid()

series = [(vA, 'forestgreen', 's', r'A: R$^2$=%0.2f, y=%0.2fx %+0.2f'),
          (vB, 'royalblue', 'o', r'B: R$^2$=%0.2f, y=%0.2fx %+0.2f'),
          (vC, 'orange', '^', r'C: R$^2$=%0.2f, y=%0.2fx %+0.2f'),]



for v,c,m,l in series:
    lobf = linregress(ven,v)
    print(l, lobf)
    
    plt.plot([0,200], lobf.slope*np.array([0,200])+lobf.intercept, color = c)

    plt.errorbar(ven, 
                    v, 
                    xerr = uncx, 
                    yerr = uncy,
                    fmt = ',',
                    ecolor = 'k',
                    elinewidth = 0.2,
                    capsize = 2.0,
                    color = c, 
                    marker = m, 
                    label = l % (lobf.rvalue**2, lobf.slope, lobf.intercept))


plt.legend()
plt.xlabel("Measured Velocity (m/s)")
plt.ylabel("Radar Velocity (m/s)")
plt.xlim([0,1.25])
plt.ylim([0,1.25])
plt.savefig('velocity_scatter.png', dpi = 300, transparent = True)
plt.show() 